<?php

/**
* Represents a Query.
*/
interface QueryToolInterface {

  /**
   * Find an array of objects
   */
  public function find();
  
  /**
   * Load items into the query
   */
  public function load($ids);
  
  /**
   * Update items
   */
  public function update($args);
   
   /**
    * Determine the edit state of the entire query object
    *
    * @return
    *   Either 'clean' or 'dirty', depending on whether any items still need 
    *   to be saved.
    */
  public function state();
}

/**
* Represents a Query
*
* Default implementation of QueryToolInterface.
*/
class QueryTool {
  /**
   * Unique ID for this query
   */
  public $uid;
  
  /**
   * A set of objects to use in queries
   */
  public $items;
  
  /**
   * The query to use
   */
  public $query;
  
  /**
   * Construct
   *
   * Essentially, we load all the helper objects here
   */
  public function __construct() {
    $this->helpers->common = new QueryToolsCommonHelper();
    $this->helpers->date = new QueryToolsDateHelper();
  }
  
  /**
   * Find an array of objects
   *
   * Default implementation of QueryToolInterface::find()
   */
  public function find(){
    return FALSE;
  }
  
  /**
   * Load items into the query
   *
   * Default implementation of QueryToolInterface::load()
   */
  public function load($ids){
    return FALSE;
  }
  
  /**
   * Update an item
   *
   * Default implementation of QueryToolInterface::update()
   */
  public function update($id, $args){
    return FALSE;
  }
  
  /**
   * Set state to normal
   *
   * Default implementation of QueryToolInterface::state_clean()
   */
  protected function state_clean($id){
    if(is_object($this->items[$id])){
      $this->items[$id]->query_tools['#state'] = QUERY_TOOLS_EDIT_STATE_NORMAL;
      return TRUE;
    }
    elseif(is_array($this->items[$id])){
      $this->items[$id]['query_tools']['#state'] = QUERY_TOOLS_EDIT_STATE_NORMAL;
      return TRUE;
    }
    return FALSE;
  }
  
  /**
   * Set state to overridden
   *
   * Default implementation of QueryToolInterface::state_dirty()
   */
  protected function state_dirty($id){
    if(is_object($this->items[$id])){
      $this->items[$id]->query_tools['#state'] = QUERY_TOOLS_EDIT_STATE_OVERRIDDEN;
      return TRUE;
    }
    elseif(is_array($this->items[$id])){
      $this->items[$id]['query_tools']['#state'] = QUERY_TOOLS_EDIT_STATE_OVERRIDDEN;
      return TRUE;
    }
    return FALSE;
  }
  
  /**
   * Default implementation of QueryToolInterface::state()
   */
   public function state(){
     foreach($this->items as $item){
       if(is_object($item)){
         if($item->query_tools['#state'] == QUERY_TOOLS_EDIT_STATE_OVERRIDDEN){
           return QUERY_TOOLS_EDIT_STATE_OVERRIDDEN;
         };
       }
       elseif(is_array($item)){
         if($item['query_tools']['#state'] == QUERY_TOOLS_EDIT_STATE_OVERRIDDEN);
         return QUERY_TOOLS_EDIT_STATE_OVERRIDDEN;
       }
     }
     
     return QUERY_TOOLS_EDIT_STATE_NORMAL;
   }
   
   /**
    * Cache
    */
  public function cache_set(){
    object_cache_set($this->type, $this->uid, $this, CACHE_TEMPORARY);
  }
}

/**
 * QueryToolsHandler
 */
abstract class QueryToolsHelper{
  
}